! Copyright 2020
!
! For a comprehensive list of the developers that contributed to these codes
! see the UK-AMOR website.
!
! This file is part of UKRmol-out (UKRmol+ suite).
!
!     UKRmol-out is free software: you can redistribute it and/or modify
!     it under the terms of the GNU General Public License as published by
!     the Free Software Foundation, either version 3 of the License, or
!     (at your option) any later version.
!
!     UKRmol-out is distributed in the hope that it will be useful,
!     but WITHOUT ANY WARRANTY; without even the implied warranty of
!     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!     GNU General Public License for more details.
!
!     You should have received a copy of the GNU General Public License
!     along with  UKRmol-out (in source/COPYING). Alternatively, you can also visit
!     <https://www.gnu.org/licenses/>.
!
!> \brief   Hard-coded parameters of MULTIDIP
!> \author  J Benda
!> \date    2020 - 2021
!>
!> This module contains mathematical and physical, as well as algorithmical parameters. The latter ones may make it
!> to the input namelist some day. At the moment their casual use is considered advanced.
!>
module multidip_params

    use precisn_gbl, only: wp

    implicit none

    integer,  parameter :: nTermsAsy = 5                !< Number of terms in expansion of the Coulomb-Hankel functions
    integer,  parameter :: nTermsPpt = 3                !< Number of terms in expansion of the exponential integral
    integer,  parameter :: nMaxPhotons = bit_size(0)    !< The limit on number of photons is `nested_cgreen_integ`
    integer,  parameter :: maxtarg = 0                  !< Maximal number of target states to calculate dipoles for (0 = all)
    logical,  parameter :: check_dipoles = .true.       !< Check that all dipole matrices in molecular_data are nonzero
    logical,  parameter :: closed_interm = .false.      !< Consider weakly closed channel in intermediate states (unfinished!)
    logical,  parameter :: coulomb_check = .true.       !< Skip integrals that cannot be asymptotically approximated well
    integer,  parameter :: num_integ_algo = 2           !< Numerical integration mode (1 = Romberg, 2 = Levin)
    integer,  parameter :: max_romb_level = 20          !< Maximal nesting level for Romberg integration
    integer,  parameter :: max_levin_level = 20         !< Maximal nesting level for Levin integration
    integer,  parameter :: cheb_order = 5               !< Order of Chebyshev interpolation used in Levin quadrature
    real(wp), parameter :: epsrel = 1e-6                !< Romberg integration relative tolerance for convergence
    real(wp), parameter :: coultol = 1e-4               !< Coulomb matching relative tolerance (decides whether to use asym. forms)
    real(wp), parameter :: closed_range = 5.0           !< Energy interval (a.u.) before threshold for inclusion of closed channels

    real(wp), parameter :: alpha = 1/137.03599907_wp    !< Fine structure constant
    real(wp), parameter :: rzero = 0
    real(wp), parameter :: rone = 1
    real(wp), parameter :: rhalf = 0.5

    complex(wp), parameter :: czero = 0
    complex(wp), parameter :: cone = 1

    character(len=1), parameter :: compt(3) = ['x', 'y', 'z']

end module multidip_params
